from __future__ import print_function
import maya.cmds as cmds
import math

def createShader(inShd, nodeType):
    if ':' in inShd:
        aiName = inShd.rsplit(':')[-1] + '_new'
    else:
        aiName = inShd + '_new'
    outNode = cmds.shadingNode(nodeType, name=aiName, asShader=True)
    return outNode

def assignToNewShader(oldShd, newShd):

    output_conns = cmds.listConnections( oldShd, d=True, s=False, c=True, plugs=True )
    output_types = []
    for output_conn in output_conns:
        output_types.append(cmds.getAttr(output_conn, type=True))

    if ':' in oldShd:
        aiName = oldShd.rsplit(':')[-1] + '_old'
    else:
        aiName = oldShd + '_old'
    
    replaceCurrentShader = replaceShaders

    try:
        cmds.rename(oldShd, aiName)
        cmds.rename(newShd, oldShd)

        newShd = oldShd
        oldShd = aiName
    except RuntimeError as err:
        return

    if output_conns:
        lenConn = len(output_conns)
        for i in range(0, lenConn, 2):
            connSplit = output_conns[i].split('.')
            if len(connSplit) > 1:

                if not cmds.attributeQuery(connSplit[1], node=connSplit[0], exists=True):
                    
                    if connSplit[1] == 'outValue':
                        if output_types[i] == 'float':
                            output_conns[i] = connSplit[0] + '.outColorR' 
                        else:
                            output_conns[i] = connSplit[0] + '.outColor' 
                    elif connSplit[1] == 'outValueX':
                        output_conns[i] = connSplit[0] + '.outColorR'
                    elif connSplit[1] == 'outValueY':
                        output_conns[i] = connSplit[0] + '.outColorG'
                    elif connSplit[1] == 'outValueZ':
                        output_conns[i] = connSplit[0] + '.outColorB'

            cmds.connectAttr(output_conns[i], output_conns[i+1], force=True)

    if replaceCurrentShader:
        cmds.delete(oldShd)

def setupConnections(inShd, fromAttr, outShd, toAttr):
    conns = cmds.listConnections( inShd + '.' + fromAttr, d=False, s=True, plugs=True )
    if conns:
        cmds.connectAttr(conns[0], outShd + '.' + toAttr, force=True)
        return True

    return False

def convertAttr(inNode, inAttr, outNode, outAttr, functionPtr = None):

    if cmds.objExists(inNode + '.' + inAttr):
        #print '\t', inAttr, ' -> ', outAttr
        
        if not setupConnections(inNode, inAttr, outNode, outAttr):
            # copy the values
            val = cmds.getAttr(inNode + '.' + inAttr)
            if functionPtr:
                val = functionPtr(val)
                
            setValue(outNode + '.' + outAttr, val)

            attrType = cmds.getAttr(inNode + '.' + inAttr, type=True)
            if attrType in ['float3']:
                subAttr = inAttr + '.' + inAttr + 'R'
                if cmds.objExists(inNode + '.' + subAttr):
                    setupConnections(inNode, subAttr, outNode, outAttr + '.' + outAttr + 'R')

                subAttr = inAttr + '.' + inAttr + 'G'
                if cmds.objExists(inNode + '.' + subAttr):
                    setupConnections(inNode, subAttr, outNode, outAttr + '.' + outAttr + 'G')

                subAttr = inAttr + '.' + inAttr + 'B'
                if cmds.objExists(inNode + '.' + subAttr):
                    setupConnections(inNode, subAttr, outNode, outAttr + '.' + outAttr + 'B')

                subAttr = inAttr + 'X'
                if cmds.objExists(inNode + '.' + subAttr):
                    setupConnections(inNode, subAttr, outNode, outAttr + 'X')

                subAttr = inAttr + 'Y'
                if cmds.objExists(inNode + '.' + subAttr):
                    setupConnections(inNode, subAttr, outNode, outAttr + 'Y')

                subAttr = inAttr + 'Z'
                if cmds.objExists(inNode + '.' + subAttr):
                    setupConnections(inNode, subAttr, outNode, outAttr + 'Z')

def setValue(attr, value):
    """Simplified set attribute function.

    @param attr: Attribute to set. Type will be queried dynamically
    @param value: Value to set to. Should be compatible with the attr type.
    """

    aType = None
    
    if cmds.objExists(attr):
        attrType = cmds.getAttr(attr, type=True)
        # temporarily unlock the attribute
        isLocked = cmds.getAttr(attr, lock=True)
        if isLocked:
            cmds.setAttr(attr, lock=False)

        # one last check to see if we can write to it
        if cmds.getAttr(attr, settable=True):
            attrType = cmds.getAttr(attr, type=True)
            
            #print value, type(value)
            
            if attrType in ['string']:
                aType = 'string'
                try:
                    cmds.setAttr(attr, value, type=aType)
                except RuntimeError as err:
                    pass
                    
            elif attrType in ['long', 'short', 'float', 'byte', 'double', 'doubleAngle', 'doubleLinear', 'bool']:
                aType = None
                try:
                    cmds.setAttr(attr, value)
                except RuntimeError as err:
                    pass    
                
            elif attrType in ['long2', 'short2', 'float2',  'double2', 'long3', 'short3', 'float3',  'double3']:
                if isinstance(value, float):
                    if attrType in ['long2', 'short2', 'float2',  'double2']:
                        value = [(value,value)]
                    elif attrType in ['long3', 'short3', 'float3',  'double3']:
                        value = [(value, value, value)]
                try:
                    cmds.setAttr(attr, *value[0], type=attrType)
                except RuntimeError as err:
                    pass    

        if isLocked:
            # restore the lock on the attr
            cmds.setAttr(attr, lock=True)

def opacityToTransparency(inShd, outShd, invert = True):
    opMap = cmds.listConnections( inShd + '.opacityR', d=False, s=True, plugs=False )
    if opMap:
        if cmds.nodeType(opMap) == "file":
            cmds.connectAttr(opMap[0] + '.outTransparency', outShd + '.transparency', force=True)
    else:
        op = cmds.getAttr(inShd + '.opacity')
        trans = None
        if invert:
            trans = [(1.0 - op[0][0], 1.0 - op[0][1], 1.0 - op[0][2])]
        else:
            trans = [(op[0][0], op[0][1], op[0][2])]
        setValue(outShd + '.transparency', trans)

def toLambert(inShd):
    outNode = createShader(inShd, 'lambert')
    convertAttr(inShd, 'base', outNode, 'diffuse')
    convertAttr(inShd, 'baseColor', outNode, 'color')
    convertAttr(inShd, 'normalCamera', outNode, 'normalCamera')

    emission = cmds.getAttr(inShd + '.emission')
    if emission > 0:
        def multByEmission(_val):
            return [(_val[0][0] * emission, _val[0][1] * emission, _val[0][2] * emission)]
        convertAttr(inShd, 'emissionColor', outNode, 'incandescence', multByEmission)
    
    opacityToTransparency(inShd, outNode, True)
    return outNode

def toPhong(inShd):
    outNode = createShader(inShd, 'phong')
    convertAttr(inShd, 'base', outNode, 'diffuse')
    convertAttr(inShd, 'baseColor', outNode, 'color')
    convertAttr(inShd, 'normalCamera', outNode, 'normalCamera')
    convertAttr(inShd, 'specularColor', outNode, 'specularColor')
    convertAttr(inShd, 'specularRoughness', outNode, 'eccentricity')

    roughness = cmds.getAttr(inShd + '.specularRoughness')
    cosinePower = 500
    if roughness > 0:
        cosinePower = (1.0 / (roughness * roughness) - 3.357) / 0.454
    print("roughness", roughness, "cosine power", cosinePower)
    setValue(outNode + '.cosinePower', cosinePower)

    emission = cmds.getAttr(inShd + '.emission')
    if emission > 0:
        def multByEmission(_val):
            return [(_val[0][0] * emission, _val[0][1] * emission, _val[0][2] * emission)]
        convertAttr(inShd, 'emissionColor', outNode, 'incandescence', multByEmission)
    
    opacityToTransparency(inShd, outNode, True)
    return outNode

def convertSelection():
    # Loops through the selection and attempts to create arnold shaders on whatever it finds
    sel = cmds.ls(sl=True)
    if sel:
        for s in sel:
            if cmds.getAttr(s, '.specular') > 0:
                ret = toPhong(s)
            else:
                ret = toLambert(s)

            if ret:
                # assign objects to the new shader
                assignToNewShader(s, ret)

def convertShaders():
    if not cmds.pluginInfo( 'mtoa', query=True, loaded=True ):
        cmds.loadPlugin('mtoa')

    convertSelection()

convertShaders()
